library example.basic_auth.server;

import 'dart:async';
import 'dart:io';
import 'dart:convert';

import 'package:jaguar/jaguar.dart';
import 'package:jaguar_auth/jaguar_auth.dart';
import 'package:jaguar_session/jaguar_session.dart';

import '../../common/models/book/book.dart';
import '../../common/user.dart';
import '../../common/books.dart';

part 'server.g.dart';

/// This route group contains login and logout authentication routes
@RouteGroup()
//You must provide an instance of SessionManager through sessionManager parameter
@WrapSessionInterceptor(makeParams: const <Symbol, MakeParam>{
  #sessionManager: const MakeParamFromMethod(#sessionManager)
})
class AuthRoutes extends _$JaguarAuthRoutes {
  @Post(path: '/login')
  //BasicAuth interceptors uses SessionManager provided by SessionInterceptor
  //to create a new session if authentication is successful
  @WrapBasicAuth(modelManager: kModelManager)
  void login(HttpRequest req) {}

  @Post(path: '/logout')
  @Input(SessionInterceptor)
  Future<Null> logout(SessionManager sessionManager) async {
    await sessionManager.clearSession();
  }

  //This is the session manager that is passed to SessionInterceptor
  CookieSessionManager sessionManager() => new CookieSessionManager();
}

/// Collection of routes that need authorization
@RouteGroup(path: '/book')
//You must provide an instance of SessionManager through sessionManager parameter
@WrapSessionInterceptor(makeParams: const <Symbol, MakeParam>{
  #sessionManager: const MakeParamFromMethod(#sessionManager)
})
//UserAuthoriser interceptor uses SessionManager provided by SessionInterceptor
//to parse and validate session information. UserAuthoriser interceptor provides
//logged-in user as Input to other interceptors and routes.
@WrapUserAuthoriser(modelManager: kModelManager)
class StudentRoutes extends _$JaguarStudentRoutes implements RequestHandler {
  @Get(path: '/all')
  //loggedinUser can be obtained from UserAuthoriser as Input
  @Input(UserAuthoriser)
  Response<String> getAllBooks(User loggedinUser) {
    List<Map> ret = books.values.map((Book book) => book.toMap()).toList();

    Response<String> resp = new Response<String>(JSON.encode(ret));
    //For demonstration we pass username of logged-in user as cookie
    resp.cookies.add(new Cookie('user', loggedinUser.username));
    return resp;
  }

  @Get(path: '/:id', headers: const {'sample-header': 'sample-value'})
  String getBook(String id) {
    Book book = books[id];
    return book.toJson();
  }

  //This is the session manager that is passed to SessionInterceptor
  CookieSessionManager sessionManager() => new CookieSessionManager();
}

@Api(path: '/api')
class LibraryApi extends Object
    with _$JaguarLibraryApi
    implements RequestHandler {
  @Group()
  final AuthRoutes auth = new AuthRoutes();

  @Group()
  final StudentRoutes student = new StudentRoutes();
}

server() async {
  LibraryApi api = new LibraryApi();

  Configuration configuration = new Configuration();
  configuration.addApi(api);

  await serve(configuration);
}
